#!/usr/bin/env node

var fs = require('fs');
var path = require('path');

var argv = require('argh').argv;
var globby = require('globby');
var Promise = require('promise');

var babel = require('babel-core');
var presets = require('babel-preset-es2015');
var uglify = require('uglify-js');

var webpack = require('webpack');
var tranformType = 'babel';
var data = {};
var plugins = [];

if (argv.w || argv.webpack) {
	tranformType = 'webpack';
}

if (argv.min) {
	plugins.push(new webpack.optimize.UglifyJsPlugin({
		compress: {
			warnings: false
		}
	}));
}

globby('**/*.es6.js').then(function(paths) {
	var dir;
	var dirData;
	var entry;
	var dirPromises = [];

	paths.forEach(prepareData);

	for (dir in data) {
		dirData = data[dir];

		for (entry in dirData) {
			;(function(dir, entry) {
				dirPromises.push(new Promise(function(resolve) {
					var entryName = data[dir][entry];
					var filePath = path.resolve(dir, entryName);
					transformFile(dir, entry, resolve);

					if (argv.serve) {
						fs.watchFile(filePath, function() {
							console.log('file changed: ', filePath);
							transformFile(dir, entry);
						});
					}
				}));
			}(dir, entry));
		}
	}

	Promise.all(dirPromises).then(function() {
		if (argv.serve) {
			console.log('waiting...');
		} else {
			console.log('done!');
		}
	});
});

function prepareData(fileName) {
	var dir = path.resolve(path.dirname(fileName));
	var fName = path.basename(fileName);
	var splitedName = fName.split('.es6.js')[0];

	if (!data[dir]) {
		data[dir] = {};
	}

	data[dir][splitedName] = '.' + path.sep + fName;
}

function transformFile(dir, entry, fileResolve) {
	var entryName = data[dir][entry];
	var filePath = path.resolve(dir, entryName);

	new Promise(function(resolve) {
		console.log('babel start: ', filePath);

		babel.transformFile(filePath, {
			presets: [presets]
		}, function(err, result) {
			fs.writeFile(path.resolve(dir, entry + '.js'), (argv.min && tranformType === 'babel' ? minifyFile(result.code) : result).code, function() {
				console.log('babel done: ', filePath);
				
				resolve();
			});
		});
	}).then(function() {
		if (tranformType === 'webpack') {
			console.log('webpack start: ', filePath);

			return new Promise(function(resolve) {
				webpack({
					context: dir,
					entry: '.' + path.sep + entry + '.js',
					output: {
						path: dir,
						filename: entry + '.js'
					},
					devtool: argv.min ? '' : 'inline-source-map',
					plugins: plugins
				}, function(err) {
					console.log('webpack done: ', err);

					resolve();
				});
			});
		}
	}).then(function() {
		if (typeof fileResolve === 'function') {
			fileResolve();
		}
	});
}

function minifyFile(fileCode) {
	return uglify.minify(fileCode, {
		fromString: true
	});
};
